/******************************************************************************

File Name:           Config_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
*******************************************************************************/
#ifndef __Config_EnergyMeter
#define __Config_EnergyMeter

/****************************************************************************
  Macro:   APPARENT POWER/ENERGY CALCULATION METHOD

 Description:   Macro deciedes the method used in apparent power/energy calculation

  Precondition:  1. For TRAINGULAR METHOD of CALCULATION
                            Active & Reactive power/enegry calculationmust be enabled in case of 
                           "Traingualr method of apparent power/energy" calculation. This method holds 
                            good with sine wave input only. in case od quasi sine or any other input signal,
                            this method will not yield correct apparent power/energy
                      
                          2. Apparent power/energy derivation as product of VOLTAGE & CURRENT
                              This methos required RMS Voltage & RMS Current calculations to be enabled.
                               This method of apparent power/energy can be applied in general across wide band of signal

  Parameters:   APPARENT_POWER_TRAINGULAR_METHOD - Traingular method of calculation for sine signal
                         APPARENT_POWER_V_I_PRODUCT_METHOD - RMS Voltage & Current product method of calculation

  Remarks: None
  ***************************************************************************/
#define APPARENT_POWER_TRAINGULAR_METHOD
//#define APPARENT_POWER_V_I_PRODUCT_METHOD

/****************************************************************************
Uncomment below macros to enable calculation of various Meter outputs
*****************************************************************************/
//Enable RMS Voltage Calculation
#define RMS_VOLTAGE_ENABLE_COMPUTATION

//Enable RMS Current Calculation
#define RMS_CURRENT_ENABLE_COMPUTATION

//Enable Active Power Calculation
#define ACTIVE_POWER_ENABLE_COMPUTATION

//Enable Reactive Power Calculation
#define REACTIVE_POWER_ENABLE_COMPUTATION

//Enable Apparent Power Calculation
#define APPARENT_POWER_ENABLE_COMPUTATION

//Enable Active Energy Calculation
#define ACTIVE_ENERGY_ENABLE_COMPUTATION

//Enable Reactive Energy Calculation
#define REACTIVE_ENERGY_ENABLE_COMPUTATION



/*********************************************************************************
************** CONDITION CHECK *************************************************
************* DONOT EDIT THIS SECTION *****************************************
**********************************************************************************/
#if defined(APPARENT_POWER_ENABLE_COMPUTATION)
	#if ( ( defined(APPARENT_POWER_TRAINGULAR_METHOD) || defined(APPARENT_POWER_V_I_PRODUCT_METHOD) )	\
	     && !defined(APPARENT_POWER_ENABLE_COMPUTATION) ) || \
		 ( defined(APPARENT_POWER_ENABLE_COMPUTATION) && ( !defined(APPARENT_POWER_TRAINGULAR_METHOD) &&	\
		 !defined(APPARENT_POWER_V_I_PRODUCT_METHOD) ) )
	#error "Enable Apparent Power/Energy Calculation with approriate method"
	#endif
	 
	#if defined(APPARENT_POWER_ENABLE_COMPUTATION) && defined(APPARENT_POWER_TRAINGULAR_METHOD) &&	\
		( !defined(ACTIVE_POWER_ENABLE_COMPUTATION) || !defined(REACTIVE_POWER_ENABLE_COMPUTATION) )
	#error "Enable Active & Reactive Power Calculation in Traingular method of Calculation" 
	#endif

	#if (defined(APPARENT_POWER_ENABLE_COMPUTATION) && defined(APPARENT_POWER_V_I_PRODUCT_METHOD)) &&	\
		( !defined(RMS_VOLTAGE_ENABLE_COMPUTATION) || !defined(RMS_CURRENT_ENABLE_COMPUTATION) )
	#error "Enable RMS Voltage & Current Calculation in V*I method of Calculation" 
	#endif
#endif	

	
#if !defined(REACTIVE_POWER_ENABLE_COMPUTATION) &&\
	!defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
#define GetFrequencyFromTimerCount(x)   
#define ProcessInstanteneousReactivePower()  
#define ReactiveGainAcrossFrequency()  
#define ReactiveGainAcrossFrequency()  
#define GetReactiveGainForLineFrequency(x)  
#endif

#if !defined(ACTIVE_POWER_ENABLE_COMPUTATION) &&\
	!defined(ACTIVE_ENERGY_ENABLE_COMPUTATION)
#define ProcessInstanteneousActivePower()   
#endif

#if !defined(ACTIVE_ENERGY_ENABLE_COMPUTATION)
#define ComputeActiveEnergy()  
#endif

#if !defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
#define ComputeReactiveEnergy()   
#endif

#if !defined(RMS_VOLTAGE_ENABLE_COMPUTATION)
#define ComputeRMSVoltage()  
#endif

#if !defined(RMS_CURRENT_ENABLE_COMPUTATION)
#define ComputeRMSCurrent()  
#endif

#if !defined(ACTIVE_POWER_ENABLE_COMPUTATION)
#define ComputeActivePower()  
#endif

#if !defined(REACTIVE_POWER_ENABLE_COMPUTATION)
#define ComputeReactivePower()  
#endif

#if !defined(APPARENT_POWER_ENABLE_COMPUTATION)
#define ComputeApparentPower()  
#endif

#if !defined(ACTIVE_ENERGY_ENABLE_COMPUTATION)
#define GainCompensateActiveEnergy()  
#endif

#if !defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
#define GainCompensateReactiveEnergy()  
#endif

#if !defined(ACTIVE_POWER_ENABLE_COMPUTATION) &&\
	!defined(ACTIVE_ENERGY_ENABLE_COMPUTATION) &&\
	!defined(REACTIVE_POWER_ENABLE_COMPUTATION) &&\
	!defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
#define ComputeSignalFrequency()  	
#endif	


#endif	//__eeprom_EnergyMeter

