/******************************************************************************

File Name:           eeprom_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
*******************************************************************************/
#ifndef __eeprom_EnergyMeter
#define __eeprom_EnergyMeter

#include "HardwareProfile_EnergyMeter.h"
#include "RegDef_EnergyMeter.h"
#include "sdadc_EnergyMeter.h"

#define  EEP_ID1	0xA2
#define  EEP_ID2	0x2A

#define EEPROM_READ_STATUS_REG	0x05						//Command for EEPROM to return the status of EEPROM
#define EEPROM_READ				0x03						//Command for EEPROM to Read from EEPROM
#define EEPROM_WRITE			0x02						//Command for EEPROM to execute write sequence
#define EEPROM_EN_WRITE			0x06						//Command for EEPROM to Write Enable

#define SAVE_ENERGY_24_HOURS	24*60*60						//24hr * 60min * 60Sec

/****************************************************************************
  Macro:   NUMBER_OF_BYTES_ENERGY_STORED_IN_EEPROM

 Description:   Macro indicates number of bytes of energy to be stored in EEPROM
                         3 x 4 bytes = 12 bytes of energy
 ***************************************************************************/
 #define  NUMBER_OF_BYTES_ENERGY_STORED_IN_EEPROM			(3*4)
 

/****************************************************************************
  Macro:   ADDRESS_OF_BYTES_ENERGY_STORED_IN_EEPROM

 Description:   Macro indicates Address in EEPROM where energy bytes have to be stored
 ***************************************************************************/
 #define  ADDRESS_OF_BYTES_ENERGY_STORED_IN_EEPROM			0x200

/****************************************************************************
  Macro:   NUMBER_OF_BYTES_CAL_REGS_STORED_IN_EEPROM

 Description:   Macro indicates number of bytes of calibration data to be stored in EEPROM
 ***************************************************************************/
 #define  NUMBER_OF_BYTES_CAL_REGS_STORED_IN_EEPROM			53
 

/****************************************************************************
  Macro:   ADDRESS_OF_BYTES_CAL_REGS_STORED_IN_EEPROM

 Description:   Macro indicates Address in EEPROM where calibration registers have to be stored
 ***************************************************************************/
 #define  ADDRESS_OF_BYTES_CAL_REGS_STORED_IN_EEPROM			0x100
 
/****************************************************************************
  Macro:   SAVE_ENERGY_TO_EEPROM_ENABLED

 Description:   Macro indicates a valid energy save to eeprom condition on power down

  Precondition:   check if it is valid to save energy on following conditions:
                            1. Energy load initialization was successful & complete configuration of meter was successul 
                            2. It is not power down condition before energy load initialzation, in which case RAM may have garbage data
                            3. Energy meter is in valid calibrated state
                            If above conditions satisfy, then save energy data on power down condition

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define SAVE_ENERGY_TO_EEPROM_ENABLED			0x66

/****************************************************************************
  Macro:   SAVE_ENERGY_TO_EEPROM_DISABLED

 Description:   Macro indicates no valid energy to save to eeprom condition on power down

  Precondition:   check if it is valid to save energy on following conditions:
                            1. Energy load initialization was successful & complete configuration of meter was successul 
                            2. It is not power down condition before energy load initialzation, in which case RAM may have garbage data
                            3. Energy meter is in valid calibrated state
                            If above conditions satisfy, then save energy data on power down condition

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define SAVE_ENERGY_TO_EEPROM_DISABLED			0x00

/****************************************************************************
  Macro:   EnableEEPROM()

 Description:   Macro selects the EEPROM chip select to enable data transfer

  Precondition:  Chip Select pin must be made output port

  Parameters:   None
  
  Returns:   None
  
  Remarks: None
  ***************************************************************************/
#define EnableEEPROM()                EM_EEPROM_CS = 0

/****************************************************************************
  Macro:   DisableEEPROM()

 Description:   Macro deselects the EEPROM chip select to enable data transfer

  Precondition:  Chip Select pin must be made output port

  Parameters:   None
  
  Returns:   None
  
  Remarks: None
  ***************************************************************************/
#define DisableEEPROM()                EM_EEPROM_CS = 1


void Init_EEPROMAddr(void);
void EEPROMAddrIncrement(WORD_VAL* Addr, UINT8* StatusWrite, UINT8* Page, UINT8* PageStatus);
void WriteEnable(void);
UINT8 ReadStatus(void);
UINT8 ByteRead(WORD_VAL* EEPROM_Addr);
void Output(UINT8 Data);
UINT8 ReadData(void);
void ByteWriteEnableandWrite(WORD_VAL* EEPROM_Addr, UINT8 DataWrite);
void ByteWrite(UINT8 DataWrite);

#endif	//__eeprom_EnergyMeter

