/******************************************************************************

File Name:           LCD_Config.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "lcd.h"

#if defined (LCD_V1_1) 
/*************************************************************************
 Function Prototype :void Config_LCD(UINT8 config1, UINT8 config2, UINT8 config3)

 Include            : lcd.h

 Description        : This function initializes the LCD controller
 
Parameters         :  config1 - This contains the parameters to be configured in the 
			// LCD Interrupt Configuration

			LCD Interrupt Configuration
				*       LCD_INT_ENABLE
				*       LCD_INT_DISABLE
				*       LCD_INT_MASK

			//LCDCON Control Register

			LCD Driver Enable
				*       LCD_DRIVER_EN
				*       LCD_DRIVER_DIS
				*       LCD_DRIVER_EN_MASK

			LCD Driver Enable in Sleep mode
				*       LCD_DRIVER_SLP_EN
				*       LCD_DRIVER_SLP_DIS
				*       LCD_DRIVER_SLP_MASK

			Clock Source Select
				*       LCD_CLK_SRC_INTRC
				*       LCD_CLK_SRC_T13CLK
				*       LCD_CLK_SRC_FOSC_4
				*       LCD_CLK_SRC_MASK

			Commons Select
				*       LCD_LMUX_COM_0
				*       LCD_LMUX_COM_0_1
				*       LCD_LMUX_COM_0_1_2
				*       LCD_LMUX_COM_0_1_2_3
				*       LCD_LMUX_MASK

			 config2 - This contains the parameters to be configured in the 
			// LCDPS-- LCD Phase register Configuration

			Waveform Type Select
				*       LCD_WAVE_TYPE_A
				*       LCD_WAVE_TYPE_B
				*       LCD_WAVE_TYPE_MASK

			Bias Mode Select
				*       LCD_BIAS_MODE_SET
				*       LCD_BIAS_MODE_CLEAR
				*       LCD_BIAS_MODE_MASK

			LCD Prescaler Select
				*       LCD_PRESCALE_1_1
				*       LCD_PRESCALE_1_2
				*       LCD_PRESCALE_1_3
				*       LCD_PRESCALE_1_4
				*       LCD_PRESCALE_1_5
				*       LCD_PRESCALE_1_6
				*       LCD_PRESCALE_1_7
				*       LCD_PRESCALE_1_8
				*       LCD_PRESCALE_1_9
				*       LCD_PRESCALE_1_10
				*       LCD_PRESCALE_1_11
				*       LCD_PRESCALE_1_12
				*       LCD_PRESCALE_1_13
				*       LCD_PRESCALE_1_14
				*       LCD_PRESCALE_1_15
				*       LCD_PRESCALE_1_16
				*       LCD_PRESCALE_MASK


			 config3 - This contains the parameters to be configured in the 
			// LCDREG- LCD Voltage Regulator register configuration

			LCD Charge Pump Enable
				*       LCD_CHARGE_PUMP_EN
				*       LCD_CHARGE_PUMP_DIS
				*       LCD_CHARGE_PUMP_EN_MASK

			Regulator Voltage Output Control
				*       LCD_REG_VOLT_2_69
				*       LCD_REG_VOLT_2_82
				*       LCD_REG_VOLT_2_95
				*       LCD_REG_VOLT_3_08
				*       LCD_REG_VOLT_3_21
				*       LCD_REG_VOLT_3_34
				*       LCD_REG_VOLT_3_47
				*       LCD_REG_VOLT_3_60
				*       LCD_REG_VOLT_MASK

			1/3 LCD Bias Enable
				*       LCD_1_3_BIAS_MODE
				*       LCD_STATIC_BIAS_MODE
				*       LCD_BIAS_MODE_MASK

			Regulator Clock Source Select
				*       LCD_REG_CLK_SRC_INTRC
				*       LCD_REG_CLK_SRC_INTOSC
				*       LCD_REG_CLK_SRC_T1OSC
				*       LCD_REG_DIS
				*       LCD_REG_CLK_MASK
                      
 Return Value       :None
 
 Remarks            :   Enables interrupt if required and clears the interrupt flag initially
*************************************************************************/

void Config_LCD(UINT8 config1, UINT8 config2, UINT8 config3)
{

	LCDCON = (config1&0xEF);	//Configure LCDCON Regsiter
	LCDPS = config2;			//Configure LCDPS Regsiter
	LCDREG = config3;			//Configure LCDREG Regsiter
	
	if( (config1&0x10) )		
	{
		PIR3bits.LCDIF=0;
		PIE3bits.LCDIE=1;		//Enable LCD Interrupt
	}
	
	Clear_LCD();		//Clear all LCDDATAx Registers
		
}

//#else
//#warning "Selected device does not support this function"
#endif
