/******************************************************************************

File Name:           LCD_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "RegDef_EnergyMeter.h"
#include "HardwareProfile_EnergyMeter.h"

//****** Global Variable that contains the BCD data to be displayed on LCD ***********

//***************** Global variable to be defined in User application that holds the data to be displayed on LCD digit ********************	
BYTE_VAL LCD_TEMP_VAL;

//******** Function Prototype declaration ***************************************

/************************************************************************************
 Function    : void Init_LCD(void)

 Description : The function initializes LCD module on controller and sets all LCD port pins drive state

 Arguments   : None

 Remarks     : Function need to be called in System initialization before LCD display update function
**************************************************************************************/
void Init_LCD(void)
{
UINT8 config1=0,config2=0,config3=0;


	//LCDCON configuration Parameters
	config1 = LCD_INT_DISABLE | LCD_DRIVER_EN | LCD_DRIVER_SLP_EN | LCD_CLK_SRC_INTRC | LCD_LMUX_COM_0_1_2_3;
	//LCDPS configuration Parameters
	config2 = LCD_WAVE_TYPE_A | LCD_BIAS_MODE_CLEAR | LCD_PRESCALE_1_5;
	//LCDREG Configuration Parameters
	config3 = LCD_CHARGE_PUMP_DIS | LCD_REG_VOLT_3_60 | LCD_1_3_BIAS_MODE | LCD_REG_CLK_SRC_INTOSC;
	
	//Configure LCD Controller
	Config_LCD(config1,config2,config3);
	
	//Configure LCD Segment registers
	//Config_LCD_Segment(0x1E,0x83,0xF8,0xA7,0x01,0x00);
	Config_LCD_Segment(0xFF,0x0F,0xF8,0xA3,0x01,0x00);//For PIC18F65J90

}


/************************************************************************************
 Function    : void DisplayDataOnLCD(UINT32 Data,DISPLAY_TYPE Type)

 Description : The function updates the display content of LCD 

 Arguments   : Data -  Data value to be displayed on LCD
		  Type -  Type of the data being displayed on LCD. It includes Active Energy, Reactive Energy, Error message

 Remarks     : Function need to be called after LCD initialization
**************************************************************************************/
void DisplayDataOnLCD(UINT32 Data,DISPLAY_TYPE Type)
{
UINT8 LCD_Array[10];

	LCD_Bin_2_BCD(Data,LCD_Array); // This below function will converted from Hex to BCD value.
	Display_LCD(LCD_Array,Type); 	//Call Function that updates the LCD with latest Energy value		
}


/************************************************************************************
 Function    : void Display_LCD(UINT8* LCD_Pattern,DISPLAY_TYPE Type)

 Description : The function updates the display content of LCD 

 Arguments   : LCD_Pattern -  pointer to LCD segment data
		  Type -  Type of the data being displayed on LCD. It includes Active Energy, Reactive Energy, Error message

 Remarks     : Function need to be called after LCD initialization
**************************************************************************************/
void Display_LCD(UINT8* LCD_Pattern,DISPLAY_TYPE Type)
{

	LCD_Display_Digit(1,*LCD_Pattern++,LCD_TEMP_VAL);		//Display Digitx from 1 to 9 on 7 segment LCD
	LCD_Display_Digit(2,*LCD_Pattern++,LCD_TEMP_VAL);		//Display Digitx from 1 to 9 on 7 segment LCD
	LCD_Display_Digit(3,*LCD_Pattern++,LCD_TEMP_VAL);		//Display Digitx from 1 to 9 on 7 segment LCD
	LCD_Display_Digit(4,*LCD_Pattern++,LCD_TEMP_VAL);		//Display Digitx from 1 to 9 on 7 segment LCD
	LCD_Display_Digit(5,*LCD_Pattern++,LCD_TEMP_VAL);		//Display Digitx from 1 to 9 on 7 segment LCD
	LCD_Display_Digit(6,*LCD_Pattern++,LCD_TEMP_VAL);		//Display Digitx from 1 to 9 on 7 segment LCD
	LCD_Display_Digit(7,*LCD_Pattern++,LCD_TEMP_VAL);		//Display Digitx from 1 to 9 on 7 segment LCD
	LCD_Display_Digit(8,*LCD_Pattern++,LCD_TEMP_VAL);		//Display Digitx from 1 to 9 on 7 segment LCD
	DIGIT_2P=1;	

	if(Type==0)
	{	
		K2 = 1;			//Display Kilo K digit on LCD
		W2 = 1;			//Display Watt W digit on LCD
		H2 = 1;			//Display Hour H digit on LCD
		
		K1 = 0;
		V1 = 0;	
		A1 = 0;
		H1 = 0;
		R1 = 0;
	}
	else
	{
		K1 = 1;
		V1 = 1;	
		A1 = 1;
		H1 = 1;
		R1 = 1;
		
		K2 = 0;			//Display Kilo K digit on LCD
		W2 = 0;			//Display Watt W digit on LCD
		H2 = 0;			//Display Hour H digit on LCD
	
	}	
}


/************************************************************************************
 Function    : void Display_Error(void)

 Description : The function updates the display content of LCD with "EEP Err" message in case of EEPROM
		   load error

 Arguments   : None

 Remarks     : Function need to be called after LCD initialization
**************************************************************************************/
void Display_Error(void)
{
UINT8 LCD_Array[10];

		LCD_Array[0] = 'r';
		LCD_Array[1] = 'r';
		LCD_Array[2] = 'E';
		LCD_Array[3] = ' ';
		LCD_Array[4] = 'P';
		LCD_Array[5] = 'E';
		LCD_Array[6] = 'E';
		LCD_Array[7] = ' ';
		LCD_Array[8] = ' ';
		LCD_Array[9] = ' ';
		Display_LCD(LCD_Array,LCD_Error);
}




