/******************************************************************************

File Name:           LCD_Seven_Seg_Pattern.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "lcd.h"

#if defined (LCD_V1_1) 
/*************************************************************************
 Function Prototype :UINT8 LCD_Seven_Seg_Pattern(UINT8 Data)

 Include            : lcd.h

 Description        : This function converts the input Data to 7 Segment pattern data
 
 Arguments          : Data -Data to be converted to 7 Segment pattern equivalent data
                      
 Return Value       :Returns 7 Segment format of data to be written into LCDDATAx register
 
 Remarks            :   None
*************************************************************************/

UINT8 LCD_Seven_Seg_Pattern(UINT8 Data)
{

	switch(Data)
	{
        case    0:					// (0x00)
            	return(LCD_7_SEGMENT_0);
				break;
		case    1:					// (0x01)
            	return(LCD_7_SEGMENT_1);
				break;
		case    2:			   		// (0x02)
            	return(LCD_7_SEGMENT_2);
				break;
		case    3:			   		// (0x03)
            	return(LCD_7_SEGMENT_3);
				break;
		case    4:			   		// (0x04)
            	return(LCD_7_SEGMENT_4);
				break;
		case    5:			   		// (0x05)
            	return(LCD_7_SEGMENT_5);
				break;
		case    6:					// (0x06)
            	return(LCD_7_SEGMENT_6);
				break;
		case    7:					// (0x07)
            	return(LCD_7_SEGMENT_7);
				break;	
		case    8:			   		// (0x08)
            	return(LCD_7_SEGMENT_8);
				break;
		case    9:			   		// (0x09)
            	return(LCD_7_SEGMENT_9);
				break;
		case    0x0A:				// (0x0A)
            	return(LCD_7_SEGMENT_A);
				break;
		case    0x0B:				// (0x0B)
            	return(LCD_7_SEGMENT_B);
				break;	
		case    0x0C:			   	// (0x0C)
            	return(LCD_7_SEGMENT_C);
				break;
		case    0x0D:				// (0x0D)
            	return(LCD_7_SEGMENT_D);
				break;
		case    0x0E:				// (0x0E)
            	return(LCD_7_SEGMENT_E);
				break;	
		case    0x0F:			   	// (0x0F)
            	return(LCD_7_SEGMENT_F);
				break;
				
        case    '0':					// (0x00)
            	return(LCD_7_SEGMENT_0);
				break;
		case    '1':					// (0x01)
            	return(LCD_7_SEGMENT_1);
				break;
		case    '2':			   		// (0x02)
            	return(LCD_7_SEGMENT_2);
				break;
		case    '3':			   		// (0x03)
            	return(LCD_7_SEGMENT_3);
				break;
		case    '4':			   		// (0x04)
            	return(LCD_7_SEGMENT_4);
				break;
		case    '5':			   		// (0x05)
            	return(LCD_7_SEGMENT_5);
				break;
		case    '6':					// (0x06)
            	return(LCD_7_SEGMENT_6);
				break;
		case    '7':					// (0x07)
            	return(LCD_7_SEGMENT_7);
				break;	
		case    '8':			   		// (0x08)
            	return(LCD_7_SEGMENT_8);
				break;
		case    '9':			   		// (0x09)
            	return(LCD_7_SEGMENT_9);
				break;
		case    'A':				// (0x0A)
            	return(LCD_7_SEGMENT_A);
				break;
		case    'b':				// (0x0B)
            	return(LCD_7_SEGMENT_B);
				break;	
		case    'C':			   	// (0x0C)
            	return(LCD_7_SEGMENT_C);
				break;
		case    'd':				// (0x0D)
            	return(LCD_7_SEGMENT_D);
				break;
		case    'E':				// (0x0E)
            	return(LCD_7_SEGMENT_E);
				break;	
		case    'F':			   	// (0x0F)
            	return(LCD_7_SEGMENT_F);
				break;
				
		case    'P':			   	// (0x10)
            	return(LCD_7_SEGMENT_P);
				break;
		case    0x11:				// (0x11)
            	return(LCD_7_SEGMENT_WL);
				break;
		case    0x12:				// (0x12)
            	return(LCD_7_SEGMENT_WR);
				break;	
		case    't':			   	// (0x13)
            	return(LCD_7_SEGMENT_t);
				break;
		case    'r':				// (0x14)
            	return(LCD_7_SEGMENT_r);
				break;
		case    'o':				// (0x15)
            	return(LCD_7_SEGMENT_o);
				break;	
		case    'u':			   	// (0x16)
            	return(LCD_7_SEGMENT_u);
				break;
		case    ' ':			   	// (0x17)
            	return(LCD_7_SEGMENT_SPACE);
				break;
		case    'H':				// (0x18)
            	return(LCD_7_SEGMENT_H);
				break;
		case    0x19:				// (0x19)
            	return(LCD_7_SEGMENT_ML);
				break;	
		case    0x1A:				// (0x1A)
            	return(LCD_7_SEGMENT_MR);
				break;		
		case    'L':				// (0x1B)
            	return(LCD_7_SEGMENT_L);
				break;
		case    'S':			   		// (0x05)
            	return(LCD_7_SEGMENT_5);
				break;				
       	default:
            	return(LCD_7_SEGMENT_SPACE);
    }//end switch(Data)
		
}

//#else
//#warning "Selected device does not support this function"
#endif
