#ifndef __LCD_H
#define __LCD_H
/******************************************************************************
 // *                   LCD PERIPHERAL LIBRARY HEADER FILE
 ******************************************************************************
 * FileName:        		lcd.h
 * Dependencies:    	See include below
 * Processor:       		PIC18F65J90
 * Compiler:        		MCC18
 * Company:         		Microchip Technology, Inc.
 *
 * Software License Agreement
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *****************************************************************************/
 
#include "pconfig.h"
#include "GenericTypeDefs.h"
#define LCD_V1_1

//***************** LCD Data register selection as Segment assignment *******************************

/*********************************************************************************************
				LCD DIGIT PATTERN USED FOR MAPPING
	7	6	5	4	3	.	2	1 		(This is representing the actual LCD digits on Panel)
**********************************************************************************************/
#define DIGIT_1A	LCDDATA0bits.S01C0
#define DIGIT_1B	LCDDATA0bits.S00C0
#define	DIGIT_1C	LCDDATA12bits.S00C2
#define	DIGIT_1D	LCDDATA18bits.S01C3
#define DIGIT_1E	LCDDATA12bits.S01C2
#define	DIGIT_1F	LCDDATA6bits.S01C1
#define	DIGIT_1G	LCDDATA6bits.S00C1
#define	DIGIT_1P	LCDDATA18bits.S00C3

#define DIGIT_2A	LCDDATA0bits.S03C0
#define DIGIT_2B	LCDDATA0bits.S02C0
#define	DIGIT_2C	LCDDATA12bits.S02C2
#define	DIGIT_2D	LCDDATA18bits.S03C3
#define DIGIT_2E	LCDDATA12bits.S03C2
#define	DIGIT_2F	LCDDATA6bits.S03C1
#define	DIGIT_2G	LCDDATA6bits.S02C1
#define	DIGIT_2P	LCDDATA18bits.S02C3

#define DIGIT_3A	LCDDATA0bits.S05C0
#define DIGIT_3B	LCDDATA0bits.S04C0
#define	DIGIT_3C	LCDDATA12bits.S04C2
#define	DIGIT_3D	LCDDATA18bits.S05C3
#define DIGIT_3E	LCDDATA12bits.S05C2
#define	DIGIT_3F	LCDDATA6bits.S05C1
#define	DIGIT_3G	LCDDATA6bits.S04C1
#define	DIGIT_3P	LCDDATA18bits.S04C3

#define DIGIT_4A	LCDDATA0bits.S07C0
#define DIGIT_4B	LCDDATA0bits.S06C0
#define	DIGIT_4C	LCDDATA12bits.S06C2
#define	DIGIT_4D	LCDDATA18bits.S07C3
#define DIGIT_4E	LCDDATA12bits.S07C2
#define	DIGIT_4F	LCDDATA6bits.S07C1
#define	DIGIT_4G	LCDDATA6bits.S06C1
#define	DIGIT_4P	LCDDATA18bits.S06C3

#define DIGIT_5A	LCDDATA1bits.S09C0
#define DIGIT_5B	LCDDATA1bits.S08C0
#define	DIGIT_5C	LCDDATA13bits.S08C2
#define	DIGIT_5D	LCDDATA19bits.S09C3
#define DIGIT_5E	LCDDATA13bits.S09C2
#define	DIGIT_5F	LCDDATA7bits.S09C1
#define	DIGIT_5G	LCDDATA7bits.S08C1
#define	DIGIT_5P	LCDDATA19bits.S08C3

#define DIGIT_6A	LCDDATA1bits.S11C0
#define DIGIT_6B	LCDDATA1bits.S10C0
#define	DIGIT_6C	LCDDATA13bits.S10C2
#define	DIGIT_6D	LCDDATA19bits.S11C3
#define DIGIT_6E	LCDDATA13bits.S11C2
#define	DIGIT_6F	LCDDATA7bits.S11C1
#define	DIGIT_6G	LCDDATA7bits.S10C1
#define	DIGIT_6P	LCDDATA19bits.S10C3

#define DIGIT_7A	LCDDATA2bits.S19C0
#define DIGIT_7B	LCDDATA3bits.S29C0
#define	DIGIT_7C	LCDDATA15bits.S29C2
#define	DIGIT_7D	LCDDATA20bits.S19C3
#define DIGIT_7E	LCDDATA14bits.S19C2
#define	DIGIT_7F	LCDDATA8bits.S19C1
#define	DIGIT_7G	LCDDATA9bits.S29C1
#define	DIGIT_7P	LCDDATA21bits.S29C3

#define DIGIT_8A	LCDDATA2bits.S21C0
#define DIGIT_8B	LCDDATA2bits.S20C0
#define	DIGIT_8C	LCDDATA14bits.S20C2
#define	DIGIT_8D	LCDDATA20bits.S21C3
#define DIGIT_8E	LCDDATA14bits.S21C2
#define	DIGIT_8F	LCDDATA8bits.S21C1
#define	DIGIT_8G	LCDDATA8bits.S20C1
#define	DIGIT_8P	LCDDATA20bits.S20C3

//#define DIGIT_9A	LCDDATA2bits.S23C0
//#define DIGIT_9B	LCDDATA2bits.S22C0
//#define	DIGIT_9C	LCDDATA14bits.S22C2
//#define	DIGIT_9D	LCDDATA20bits.S23C3
//#define DIGIT_9E	LCDDATA14bits.S23C2
//#define	DIGIT_9F	LCDDATA8bits.S23C1
//#define	DIGIT_9G	LCDDATA8bits.S22C1
//#define	DIGIT_9P	LCDDATA23bits.S40C3
/////End of Change

#define	K2			LCDDATA9bits.S31C1
#define	W2			LCDDATA22bits.S32C3
#define	H2			LCDDATA21bits.S31C3

#define K1			LCDDATA4bits.S32C0
#define	V1			LCDDATA3bits.S31C0
#define	A1			LCDDATA16bits.S32C2
#define	H1			LCDDATA10bits.S32C1
#define	R1			LCDDATA15bits.S31C2

//************************ LCD 7 Segment Look Up Table data *******************************************************
#define LCD_7_SEGMENT_0			0b00111111	                        // 7 Segment Pattern for Zero
#define LCD_7_SEGMENT_1			0b00000110	                        // 7 Segment Pattern for One
#define LCD_7_SEGMENT_2			0b01011011	                        // 7 Segment Pattern for Two
#define LCD_7_SEGMENT_3			0b01001111	                        // 7 Segment Pattern for Three
#define LCD_7_SEGMENT_4			0b01100110	                        // 7 Segment Pattern for Four
#define LCD_7_SEGMENT_5			0b01101101	                        // 7 Segment Pattern for Five
#define LCD_7_SEGMENT_6			0b01111101	                        // 7 Segment Pattern for Six
#define LCD_7_SEGMENT_7			0b00000111	                        // 7 Segment Pattern for Seven
#define LCD_7_SEGMENT_8			0b01111111	                        // 7 Segment Pattern for Eight
#define LCD_7_SEGMENT_9			0b01101111	                        // 7 Segment Pattern for Nine
#define LCD_7_SEGMENT_A			0b01110111	                        // 7 Segment Pattern for A
#define LCD_7_SEGMENT_B			0b01111100	                        // 7 Segment Pattern for B
#define LCD_7_SEGMENT_C			0b00111001	                        // 7 Segment Pattern for C
#define LCD_7_SEGMENT_D			0b01011110	                        // 7 Segment Pattern for D
#define LCD_7_SEGMENT_E			0b01111001	                        // 7 Segment Pattern for E
#define LCD_7_SEGMENT_F			0b01110001	                        // 7 Segment Pattern for F
#define LCD_7_SEGMENT_P			0b01110011	                        // 7 Segment Pattern for P (0x10)
#define LCD_7_SEGMENT_WL		0b00111100	                        // 7 Segment Pattern for left side of W (0x11) "Wl" - "|  "
																	//								         |_|
																								   
#define LCD_7_SEGMENT_WR		0b00011110	                        // 7 Segment Pattern for right side of W (0x12) "Wr" - "  |"
																	//									|_|
																									
#define LCD_7_SEGMENT_t			0b01111000	                        // 7 Segment Pattern for t (0x13)
#define LCD_7_SEGMENT_r			0b01010000	                        // 7 Segment Pattern for r (0x14)
#define LCD_7_SEGMENT_o			0b01011100	                        // 7 Segment Pattern for o (0x15)
#define LCD_7_SEGMENT_u			0b00011100	                        // 7 Segment Pattern for u (0x16)
#define LCD_7_SEGMENT_SPACE			0b00000000	                        // 7 Segment Pattern for space (0x17)
#define LCD_7_SEGMENT_H			0b01110110	                        // 7 Segment Pattern for H (0x18)
#define LCD_7_SEGMENT_ML		0b00110011	                        // 7 Segment Pattern for left side of M (0x19) "Ml" - " _ "
																	//								         | |
																	//								         |
																								   
#define LCD_7_SEGMENT_MR		0b00100111	                        // 7 Segment Pattern for right side of M (0x1A) "Mr" - " _ "
																	//									 | |
																	//									 |
#define LCD_7_SEGMENT_L			0b00111000	                        // 7 Segment Pattern for L (0x1B)
#define LCD_7_SEGMENT_POINT		0b10000000	                        // 7 Segment Pattern for "." point
	


/* PIC18 LCD peripheral library. */

#if	defined (LCD_V1_1)

/* LCD Interrupt Configuration */

#define LCD_INT_ENABLE						0x10	//Enable LCD Interrupt
#define LCD_INT_DISABLE						0x00	//Disable LCD Interrupt
#define LCD_INT_MASK						(~LCD_INT_ENABLE)	//Mask LCD Interrupt selection

/*LCDCON Control Register*/

#define LCD_DRIVER_EN						0x80	//Enable LCD driver
#define LCD_DRIVER_DIS						0x00	//Disable LCD driver
#define LCD_DRIVER_EN_MASK					(~LCD_DRIVER_EN)	//Mask LCD Enable/Disable selection bit

#define LCD_DRIVER_SLP_EN					0x00	//Enable LCD driver in Sleep Mode
#define LCD_DRIVER_SLP_DIS					0x40	//Disable LCD driver in Sleep Mode
#define LCD_DRIVER_SLP_MASK					(~LCD_DRIVER_SLP_EN)	//Mask LCD Sleep Enable/Disable selection bit

#define LCD_CLK_SRC_INTRC					0x0C	//LCD Clock source as INTRC
#define LCD_CLK_SRC_T13CLK					0x04	//LCD Clock source as T13CLK
#define LCD_CLK_SRC_FOSC_4					0x00	//LCD Clock source as FOSC/4
#define LCD_CLK_SRC_MASK					(~LCD_CLK_SRC_INTRC)	//Mask LCD Clock selection bit

#define LCD_LMUX_COM_0						0x00	//LCD MUX type as COM0 with Static Bias mode
#define LCD_LMUX_COM_0_1					0x01	//LCD MUX type as COM0 & COM1 with 1/2 mode
#define LCD_LMUX_COM_0_1_2					0x02	//LCD MUX type as COM0, COM1 & COM2 with 1/3 mode
#define LCD_LMUX_COM_0_1_2_3				0x03	//LCD MUX type as COM0, COM1, COM2 & COM3 with 1/4 mode
#define LCD_LMUX_MASK						(~LCD_LMUX_COM_0_1_2_3)	//Mask LCD Common selection bit

/* LCDPS-- LCD Phase register Configuration */

#define LCD_WAVE_TYPE_A						0x00	//LCD Wave form type selection as TYPE-A
#define LCD_WAVE_TYPE_B						0x80	//LCD Wave form type selection as TYPE-B
#define LCD_WAVE_TYPE_MASK					(~LCD_WAVE_TYPE_B)	//Mask LCD Wave Type selection bit

#define LCD_BIAS_MODE_SET					0x40	//Set LCD Bias Mode
#define LCD_BIAS_MODE_CLEAR					0x00	//Clear LCD Bias Mode
#define LCD_BIAS_MODE_MASK					(~LCD_BIAS_MODE_SET)	//Mask LCD Bias Mode selection bit

#define LCD_PRESCALE_1_1					0x00	//LCD Prescaler set to 1:1
#define LCD_PRESCALE_1_2					0x01	//LCD Prescaler set to 1:2
#define LCD_PRESCALE_1_3					0x02	//LCD Prescaler set to 1:3
#define LCD_PRESCALE_1_4					0x03	//LCD Prescaler set to 1:4
#define LCD_PRESCALE_1_5					0x04	//LCD Prescaler set to 1:5
#define LCD_PRESCALE_1_6					0x05	//LCD Prescaler set to 1:6
#define LCD_PRESCALE_1_7					0x06	//LCD Prescaler set to 1:7
#define LCD_PRESCALE_1_8					0x07	//LCD Prescaler set to 1:8
#define LCD_PRESCALE_1_9					0x08	//LCD Prescaler set to 1:9
#define LCD_PRESCALE_1_10					0x09	//LCD Prescaler set to 1:10
#define LCD_PRESCALE_1_11					0x0A	//LCD Prescaler set to 1:11
#define LCD_PRESCALE_1_12					0x0B	//LCD Prescaler set to 1:12
#define LCD_PRESCALE_1_13					0x0C	//LCD Prescaler set to 1:13
#define LCD_PRESCALE_1_14					0x0D	//LCD Prescaler set to 1:14
#define LCD_PRESCALE_1_15					0x0E	//LCD Prescaler set to 1:15
#define LCD_PRESCALE_1_16					0x0F	//LCD Prescaler set to 1:16
#define LCD_PRESCALE_MASK					(~LCD_PRESCALE_1_16)	//Mask LCD Prescaler selection bit

/* LCDREG- LCD Voltage Regulator register configuration */

#define LCD_CHARGE_PUMP_EN					0x40	//Enable LCD Charge Pump
#define LCD_CHARGE_PUMP_DIS					0x00	//Disable LCD Charge Pump
#define LCD_CHARGE_PUMP_EN_MASK				(~LCD_CHARGE_PUMP_EN)	//Mask LCD Charge Pump Enable bit

#define LCD_REG_VOLT_2_69					0x00	//LCD Regulator Volatge Output is set to 2.69V with offset of 0.91V on LCDBIAS0
#define LCD_REG_VOLT_2_82					0x08	//LCD Regulator Volatge Output is set to 2.82V with offset of 0.78V on LCDBIAS0
#define LCD_REG_VOLT_2_95					0x10	//LCD Regulator Volatge Output is set to 2.95V with offset of 0.65V on LCDBIAS0
#define LCD_REG_VOLT_3_08					0x18	//LCD Regulator Volatge Output is set to 3.08V with offset of 0.52V on LCDBIAS0
#define LCD_REG_VOLT_3_21					0x20	//LCD Regulator Volatge Output is set to 3.21V with offset of 0.39V on LCDBIAS0
#define LCD_REG_VOLT_3_34					0x28	//LCD Regulator Volatge Output is set to 3.34V with offset of 0.26V on LCDBIAS0
#define LCD_REG_VOLT_3_47					0x30	//LCD Regulator Volatge Output is set to 3.47V with offset of 0.13V on LCDBIAS0
#define LCD_REG_VOLT_3_60					0x38	//LCD Regulator Volatge Output is set to 3.60V with offset of 0.00V on LCDBIAS0
#define LCD_REG_VOLT_MASK					(~LCD_REG_VOLT_3_60)	//Mask LCD Regulator Volatge Output Control bit

#define LCD_1_3_BIAS_MODE					0x04	//Regulator output supports 1/3 LCD Bias mode
#define LCD_STATIC_BIAS_MODE				0x00	//Regulator output supports static LCD Bias mode
#define LCD_1_3_BIAS_MODE_MASK				(~LCD_1_3_BIAS_MODE)	//Mask 1/3 LCD Bias Enable bit

#define LCD_REG_CLK_SRC_INTRC				0x03	//LCD Regulator Clock source as INTRC
#define LCD_REG_CLK_SRC_INTOSC				0x02	//LCD Regulator Clock source as INTOSC 8MHz
#define LCD_REG_CLK_SRC_T1OSC				0x01	//LCD Regulator Clock source as Timer 1 Oscillator -- T1OSC
#define LCD_REG_DIS							0x00	//LCD Regulator is Disabled
#define LCD_REG_CLK_MASK					(~LCD_REG_CLK_SRC_INTRC)	//Mask LCD Regulator Clock Selection bits

/*********************************************************************
 Macro       : LCD_Write_Failure_Status()

 Include     : lcd.h
 
 Description : This macro returns the status of write failure error bit in LCDCON
 
 Arguments   :  None
 
 Returns:	Returns the status of write failure error bit in LCDCON
 
 Remarks     : Returns 1 : When there is an error in writing to LCDDATx register while WA is clear
			    0 : When successful write to LCDDATx register with WA set
**********************************************************************/   
#define LCD_Write_Failure_Status()						(LCDCONbits.WERR)


/*********************************************************************
 Macro       : LCD_Driver_Active_Status()

 Include     : lcd.h
 
 Description : This macro returns the status of LCDA bit in LCDPS register indiacting the status of LCD Driver
 
 Arguments   :  None
 
 Returns:	Returns the status of LCD Driver
 
 Remarks     : Returns 1 : When LCDA bit in LCDPS is set indicating LCD Driver is active
			    0 : When LCDA bit in LCDPS is cleared indicating LCD Driver is inactive
**********************************************************************/   
#define LCD_Driver_Active_Status()						(LCDPSbits.LCDA)


/*********************************************************************
 Macro       : LCD_Write_Allow_Status()

 Include     : lcd.h
 
 Description : This macro returns the status of WA bit in LCDPS register indiacting wether write to LCDDATx register is allowed or not
 
 Arguments   :  None
 
 Returns:	Returns the status of LCD Write allow WA bit
 
 Remarks     : Returns 1 : When WA bit in LCDPS is set indicating LCDDATx registers can be written
			    0 : When WA bit in LCDPS is cleared indicating LCDDATx registers cannot be written
**********************************************************************/   
#define LCD_Write_Allow_Status()						(LCDPSbits.WA)


/*********************************************************************
 Macro       : LCD_Segment_Enable(Segment,Value)

 Include     : lcd.h
 
 Description : This macro Enables the Segments by writing into LCDSEx Register
 
 Arguments   :  Segment - Segment number which has to be enables
		    Value - Value to be written to LCDSEx register to enable respective Segment lines
 
 Returns:	None
 
 Remarks     : None
**********************************************************************/   
#define LCD_Segment_Enable(Segment,Value)						LCDSE##Segment = Value


/*********************************************************************
 Macro       : LCD_Display_Digit(Digit,LCDData)

 Include     : lcd.h
 
 Description : This macro Displays the data on 7 Segment LCD
 
 Arguments   :  Digit - Digit on which the data has to be displayed
		    LCDData - Hex value of the data to be displayed on LCD's 'n'th digit
			*       0  --  7 Segment Pattern for Zero
			*       1  --  7 Segment Pattern for One
			*       2  --  7 Segment Pattern for Two
			*       3  --  7 Segment Pattern for Three
			*       4  --  7 Segment Pattern for Four
			*       5  --  7 Segment Pattern for Five
			*       6  --  7 Segment Pattern for Six
			*       7  --  7 Segment Pattern for Seven
			*       8  --  7 Segment Pattern for Eight
			*       9  --  7 Segment Pattern for Nine
			*       0x0A  --  7 Segment Pattern for A
			*       0x0B  --  7 Segment Pattern for B
			*       0x0C  --  7 Segment Pattern for C
			*       0x0D  --  7 Segment Pattern for D
			*       0x0E  --  7 Segment Pattern for E
			*       0x0F  --  7 Segment Pattern for F
			*       0x10 Segment Pattern for P (0x10)
			*       0x11  --  7 Segment Pattern for left side of W (0x11) "Wl" - "|  "
													      |_|
																													   
			*       0x12  --  7 Segment Pattern for right side of W (0x12) "Wr" - "  |"
													         |_|
																														
			*       0x13  --  7 Segment Pattern for t (0x13)
			*       0x14  --  7 Segment Pattern for r (0x14)
			*       0x15  --  7 Segment Pattern for o (0x15)
			*       0x16  --  7 Segment Pattern for u (0x16)
			*       0x17  --  7 Segment Pattern for space (0x17)
			*       0x18  --  7 Segment Pattern for H (0x18)
			*       0x19  --  7 Segment Pattern for left side of M (0x19) "Ml" - " _ "
													       | |
													       |
																													   
			*       0x1A  --  7 Segment Pattern for right side of M (0x1A) "Mr" - " _ "
													          | |
													            |
			*       0x1B  --  7 Segment Pattern for L (0x1B)	
		ptr - Srtucture of type BYTE_VAL

 Returns:	None
 
 Remarks     :1.  	LCDData to be displayed on LCD is the raw hex data to be passed which is then converted to BCD form
			and the LCD 7 segment equivalent data to passed on to LCD is then obtained and this data is loaded 
			into LCDDATAx register
		2. 	LCDData to be passed must be a single digit. It takes onlu lower nibble of data to be displayed	
**********************************************************************/   
#define LCD_Display_Digit(Digit,LCDData,ptr)		{	ptr.Val = LCD_Seven_Seg_Pattern(LCDData);\
															DIGIT_##Digit##A=ptr.bits.b0;\
															DIGIT_##Digit##B=ptr.bits.b1;\
															DIGIT_##Digit##C=ptr.bits.b2;\
															DIGIT_##Digit##D=ptr.bits.b3;\
															DIGIT_##Digit##E=ptr.bits.b4;\
															DIGIT_##Digit##F=ptr.bits.b5;\
															DIGIT_##Digit##G=ptr.bits.b6;\
															DIGIT_##Digit##P=ptr.bits.b7; }
													


void Config_LCD(UINT8 config1, UINT8 config2, UINT8 config3);

void Config_LCD_Segment(UINT8 segment0, UINT8 segment1, UINT8 segment2,
						UINT8 segment3, UINT8 segment4, UINT8 segment5);

void Clear_LCD(void);

UINT8 LCD_Seven_Seg_Pattern(UINT8 Data);

#endif

#endif
