/******************************************************************************

File Name:           ApparentPower_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "MeteringEngine_EnergyMeter.h"



#if defined(APPARENT_POWER_ENABLE_COMPUTATION)
/************************************************************************************
 Function    : void ComputeApparentPower(void)

 Description : The function computes Apparent Power

 Arguments   : None
 
 Return         : Updates Apparent Power variable

 Remarks     : Function need to be used along with Active Power & Reactive Power computation functions
		in case of Traingular method & RMS Voltage & RMS Current calculation function in case
		of V * I Product method of Apparent power computation
**************************************************************************************/
void ComputeApparentPower(void)
{
	if(IsComputationCycleCompleted())
	{
		QWORD_VAL MUL_arg1,MUL_arg2;

		#if defined(APPARENT_POWER_TRAINGULAR_METHOD)
			MUL_arg1.word.LW = POWER_ACT >> 12;										//Scaling down to match the lenght within the sqrt function max limit
			MUL_arg2.word.LW = POWER_REACT >> 12;									//Scaling down to match the lenght within the sqrt function max limit
			MUL_arg1.dword.HD = (UINT32)MUL_arg1.word.LW*MUL_arg1.word.LW;			//Square the Active Power
			MUL_arg2.dword.HD = (UINT32)MUL_arg2.word.LW*MUL_arg2.word.LW;			//Square the Reactive Power
			MUL_arg2.dword.HD = MUL_arg2.dword.HD + MUL_arg1.dword.HD;				// App_Power=SQRT(Act_Power^2+React_Power^2)
			POWER_APP = (UINT32)sqrt((float)MUL_arg2.dword.HD);							//Square root is obtained
			POWER_APP = (UINT32)POWER_APP*4097;										//Scale up by same factor as by which it was scaled down
		#elif defined(APPARENT_POWER_V_I_PRODUCT_METHOD)
			POWER_APP = (UINT32)V_RMS * I_RMS;
		#endif
	}	
}
#endif


