/******************************************************************************

File Name:           CurrentRMS_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "MeteringEngine_EnergyMeter.h"



#if defined(RMS_CURRENT_ENABLE_COMPUTATION)
/************************************************************************************
 Function    : void ComputeRMSCurrent(void)

 Description : The function computes RMS Current

 Arguments   : None
 
 Return         : Updates RMS Current variable

 Remarks     : None
**************************************************************************************/
void ComputeRMSCurrent(void)
{
UINT8 ShiftCount;
    
	ADD_64(&Accumulate_Raw_RMS_Current,&Summation_RMS_Square_Current,&Accumulate_Raw_RMS_Current);		//Add sum Current^2 calculated over last line cycle to running sum for this LINE_CYC group.

	if(IsComputationCycleCompleted())
	{
	    //Check if new absolute maximums (absolute value of top byte of current/Current ADC results) over the last line cycle are greater than those for this LINE_CYC group so far, replace if so.
		if(CurrentABSMaxAmp > I_ABS_MAX)
	        I_ABS_MAX = CurrentABSMaxAmp;

			
		//Update the volatge samples squared value before computing the avarage
	    RAW2_I_RMS.dword.LD = Accumulate_Raw_RMS_Current.dword.LD;
		RAW2_I_RMS.dword.HD = Accumulate_Raw_RMS_Current.dword.HD;
	    //Multply the squared current samples value by the number of input cycles per computaion cycles value in order to calculate  the  avarage before finally calculating square root for RMS value
		//------------------------------------------------------------------------------
		ShiftCount = 9-LastLineCycCnt;
	    while(ShiftCount--)
			Left_Shift_64(RAW2_I_RMS);
		//------------------------------------------------------------------------------
		//Get the square root of squared value Current samples along with adding the offset to get the RMS value of Current
		square_val.dword.LD = RAW2_I_RMS.dword.LD;
		square_val.dword.HD = RAW2_I_RMS.dword.HD;
		sq_offset = OFFSET_I_RMS;
	    RAW_I_RMS =Compute_RMS();

	    I_RMS =(UINT16) (((UINT32)RAW_I_RMS * GAIN_I_RMS)>>15);		//Multiply RMS value of Current with gain factor to make it equivalent to the Line Current RMS value. Neglect the lower 2 bytes of value and consider only 2 bytes of final upper bytes
	}
}
#endif

