/******************************************************************************

File Name:           ReactivePower_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "MeteringEngine_EnergyMeter.h"


#if defined(REACTIVE_POWER_ENABLE_COMPUTATION)

static QWORD_VAL react_power_after_gain;


/************************************************************************************
 Function    : void ComputeReactivePower(void)

 Description : The function computes Reactive Power

 Arguments   : None
 
 Return         : Updates Reactive Power variable

 Remarks     : Function need to be used along with "ProcessInstanteneousReactivePower" function only
**************************************************************************************/
void ComputeReactivePower(void)
{

	if(IsComputationCycleCompleted())
	{
		DWORD_VAL raw_power;
		
		RAW_POWER_REACT.dword.LD = Average_Reactive_Power.dword.LD;					//Load phase power at the end of the computation cycle to raw power
		RAW_POWER_REACT.dword.HD = Average_Reactive_Power.dword.HD;
		
		//Check if the Power phase offset that is calculated in GUI and sotred in offset register is negetive or postive offset
		if(OFFSET_POWER_REACT & 0x80000000)
		{
			Reactive_Energy_Raw_Over_One_Cycle.dword.LD = (~((UINT32)OFFSET_POWER_REACT))+1;
			Reactive_Energy_Raw_Over_One_Cycle.dword.HD = 0;				//In case of negetive offset,sign extend for 4 byte equivalency
		}
		else
		{
			Reactive_Energy_Raw_Over_One_Cycle.dword.LD = OFFSET_POWER_REACT;	
			Reactive_Energy_Raw_Over_One_Cycle.dword.HD = 0;						//In case of positive offset, sign extend
		}

		if(LastLineCycCnt)
		{
			UINT8 loop_cnt;
			
			loop_cnt = LastLineCycCnt;
			while(loop_cnt--)
			{
				Left_Shift_64(Reactive_Energy_Raw_Over_One_Cycle);
			}

		}

		if( ((RAW_POWER_REACT.dword.HD == Reactive_Energy_Raw_Over_One_Cycle.dword.HD)&&
		     (RAW_POWER_REACT.dword.LD >=Reactive_Energy_Raw_Over_One_Cycle.dword.LD)) || 
			 (RAW_POWER_REACT.dword.HD > Reactive_Energy_Raw_Over_One_Cycle.dword.HD) )
			SUB_64(&RAW_POWER_REACT,&Reactive_Energy_Raw_Over_One_Cycle,&Reactive_Energy_Raw_Over_One_Cycle);		//Add offset value to the line cycle power	
		else
		{
			Reactive_Energy_Raw_Over_One_Cycle.dword.LD = RAW_POWER_REACT.dword.LD;
			Reactive_Energy_Raw_Over_One_Cycle.dword.HD = RAW_POWER_REACT.dword.HD;	
		}

		{
			UINT8 loop_cnt;
			
			loop_cnt = 9-LastLineCycCnt;
			while(loop_cnt--)
			{	
				Left_Shift_64(Reactive_Energy_Raw_Over_One_Cycle);
			}	
		}
		
		//Load the offset corrceted value of energy to temporary variable to multiply it with GLSB factor. Temporary variables are used to avoid intermediate value being read by GUI during the time of calibration and energy disaply on the GUI
		raw_power.word.LW = Reactive_Energy_Raw_Over_One_Cycle.word.HW;
		raw_power.word.HW = Reactive_Energy_Raw_Over_One_Cycle.word.UW;

		react_power_after_gain.dword.LD = 0;					//Clear the multiplication result storing variable as the MUL_S_32 demands it
		react_power_after_gain.dword.HD = 0;

		//Multiply active power with the GLSB factor to adjust the gain
	    MUL_S_32(VAR_POW_GAIN_FREQUENCY,raw_power.Val,(INT16*)&react_power_after_gain);

	   	//Neglect the lower 10 bits of the gain adjusted power as its value will be insignificant
		//-------------------------------------------------------------------------
		Left_Shift_64(react_power_after_gain);
		Left_Shift_64(react_power_after_gain);

		react_power_after_gain.word.LW = react_power_after_gain.word.HW;
		react_power_after_gain.word.HW = react_power_after_gain.word.UW;

		POWER_REACT = (UINT32)(react_power_after_gain.dword.LD);		//Update the Reactive power regsiter
	}
}
#endif
