/******************************************************************************

File Name:           Simulation_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1
*******************************************************************************/
#include "p18cxxx.h"
#include "GenericTypeDefs.h"

/*********************************************************************************
Below array of data provides SINE WAVE samples that can be used in Simulation of input Signal
for energy meter for any testing purpose
**********************************************************************************/

//240*root(2) Amplitude
//pure sine 60Hz
rom const INT16 VOLT_SAMP[128] = {
0,
2873,
5687,
8385,
10913,
13218,
15254,
16980,
18360,
19367,
19979,
20184,
19979,
19367,
18360,
16980,
15254,
13218,
10913,
8385,
5687,
2873,
0,
-2873,
-5687,
-8385,
-10913,
-13218,
-15254,
-16980,
-18360,
-19367,
-19979,
-20184,
-19979,
-19367,
-18360,
-16980,
-15254,
-13218,
-10913,
-8385,
-5687,
-2873,
0,
2873,
5687,
8385,
10913,
13218,
15254,
16980,
18360,
19367,
19979,
20184,
19979,
19367,
18360,
16980,
15254,
13218,
10913,
8385,
5687,
2873,
0,
-2873,
-5687,
-8385,
-10913,
-13218,
-15254,
-16980,
-18360,
-19367,
-19979,
-20184,
-19979,
-19367,
-18360,
-16980,
-15254,
-13218,
-10913,
-8385,
-5687,
-2873,
0,
2873,
5687,
8385,
10913,
13218,
15254,
16980,
18360,
19367,
19979,
20184,
19979,
19367,
18360,
16980,
15254,
13218,
10913,
8385,
5687,
2873,
0,
-2873,
-5687,
-8385,
-10913,
-13218,
-15254,
-16980,
-18360,
-19367,
-19979,
-20184,
-19979,
-19367,
-18360,
-16980,
-15254,
-13218,

};

//pure sine 50Hz
//rom const INT16 VOLT_SAMP[128] = {
//0,
//2429,
//4823,
//7147,
//9367,
//11451,
//13368,
//15091,
//16594,
//17857,
//18859,
//19588,
//20032,
//20184,
//20044,
//19611,
//18894,
//17902,
//16650,
//15155,
//13441,
//11531,
//9453,
//7238,
//4918,
//2526,
//97,
//-2333,
//-4729,
//-7056,
//-9281,
//-11370,
//-13295,
//-15026,
//-16539,
//-17811,
//-18825,
//-19564,
//-20020,
//-20184,
//-20055,
//-19634,
//-18928,
//-17947,
//-16705,
//-15220,
//-13513,
//-11611,
//-9539,
//-7329,
//-5012,
//-2623,
//-195,
//2236,
//4634,
//6964,
//9194,
//11290,
//13221,
//14961,
//16483,
//17765,
//18789,
//19540,
//20007,
//20183,
//20066,
//19657,
//18962,
//17991,
//16759,
//15283,
//13585,
//11690,
//9625,
//7420,
//5106,
//2719,
//292,
//-2139,
//-4539,
//-6873,
//-9107,
//-11209,
//-13148,
//-14895,
//-16426,
//-17719,
//-18753,
//-19515,
//-19994,
//-20182,
//-20076,
//-19678,
//-18995,
//-18035,
//-16813,
//-15347,
//-13657,
//-11769,
//-9710,
//-7510,
//-5201,
//-2816,
//-390,
//2042,
//4444,
//6781,
//9020,
//11128,
//13074,
//14829,
//16370,
//17672,
//18717,
//19490,
//19980,
//20180,
//20086,
//19700,
//19028,
//18079,
//16867,
//15410,
//13729,
//11848,
//9796,
//7600
//
//
//};

//alternate +ve & -ve
//rom const INT16 CURR_SAMP[128] = {
//0,
//2873,
//5687,
//8385,
//10913,
//13218,
//15254,
//16980,
//18360,
//19367,
//19979,
//20184,
//-20184,
//-19979,
//-19367,
//-18360,
//-16980,
//-15254,
//-13218,
//-10913,
//-8385,
//-5687,
//-2873,
//0,
//2873,
//5687,
//8385,
//10913,
//13218,
//15254,
//16980,
//18360,
//19367,
//19979,
//20184,
//-19979,
//-19367,
//-18360,
//-16980,
//-15254,
//-13218,
//-10913,
//-8385,
//-5687,
//-2873,
//0,
//2873,
//5687,
//8385,
//10913,
//13218,
//15254,
//16980,
//18360,
//19367,
//19979,
//20184,
//
//
//19979,
//19367,
//18360,
//16980,
//15254,
//13218,
//10913,
//8385,
//5687,
//2873,
//0,
//-2873,
//-5687,
//-8385,
//-10913,
//-13218,
//-15254,
//-16980,
//-18360,
//-19367,
//-19979,
//-20184,
//19979,
//19367,
//18360,
//16980,
//15254,
//13218,
//10913,
//8385,
//5687,
//2873,
//0,
//-2873,
//-5687,
//-8385,
//-10913,
//-13218,
//-15254,
//-16980,
//-18360,
//-19367,
//-19979,
//
//
//19979,
//19367,
//18360,
//16980,
//15254,
//13218,
//10913,
//8385,
//5687,
//2873,
//0,
//-2873,
//-5687,
//-8385,
//-10913,
//-13218,
//-15254,
//-16980,
//-18360,
//-19367,
//-19979,
//-20184,
//-19979,
//-19367,
//-18360,
//-16980,
//-15254,
//-13218,
//
//};
//

//pure sine wave 50Hz
//rom const INT16 CURR_SAMP[128] = {
//0,
//2429,
//4823,
//7147,
//9367,
//11451,
//13368,
//15091,
//16594,
//17857,
//18859,
//19588,
//20032,
//20184,
//20044,
//19611,
//18894,
//17902,
//16650,
//15155,
//13441,
//11531,
//9453,
//7238,
//4918,
//2526,
//97,
//-2333,
//-4729,
//-7056,
//-9281,
//-11370,
//-13295,
//-15026,
//-16539,
//-17811,
//-18825,
//-19564,
//-20020,
//-20184,
//-20055,
//-19634,
//-18928,
//-17947,
//-16705,
//-15220,
//-13513,
//-11611,
//-9539,
//-7329,
//-5012,
//-2623,
//-195,
//2236,
//4634,
//6964,
//9194,
//11290,
//13221,
//14961,
//16483,
//17765,
//18789,
//19540,
//20007,
//20183,
//20066,
//19657,
//18962,
//17991,
//16759,
//15283,
//13585,
//11690,
//9625,
//7420,
//5106,
//2719,
//292,
//-2139,
//-4539,
//-6873,
//-9107,
//-11209,
//-13148,
//-14895,
//-16426,
//-17719,
//-18753,
//-19515,
//-19994,
//-20182,
//-20076,
//-19678,
//-18995,
//-18035,
//-16813,
//-15347,
//-13657,
//-11769,
//-9710,
//-7510,
//-5201,
//-2816,
//-390,
//2042,
//4444,
//6781,
//9020,
//11128,
//13074,
//14829,
//16370,
//17672,
//18717,
//19490,
//19980,
//20180,
//20086,
//19700,
//19028,
//18079,
//16867,
//15410,
//13729,
//11848,
//9796,
//7600
//
//};
//

/* //pure sine wave
rom const INT16 CURR_SAMP[128] = {
0,
2873,
5687,
8385,
10913,
13218,
15254,
16980,
18360,
19367,
19979,
20184,
19979,
19367,
18360,
16980,
15254,
13218,
10913,
8385,
5687,
2873,
0,
-2873,
-5687,
-8385,
-10913,
-13218,
-15254,
-16980,
-18360,
-19367,
-19979,
-20184,
-19979,
-19367,
-18360,
-16980,
-15254,
-13218,
-10913,
-8385,
-5687,
-2873,
0,
2873,
5687,
8385,
10913,
13218,
15254,
16980,
18360,
19367,
19979,
20184,
19979,
19367,
18360,
16980,
15254,
13218,
10913,
8385,
5687,
2873,
0,
-2873,
-5687,
-8385,
-10913,
-13218,
-15254,
-16980,
-18360,
-19367,
-19979,
-20184,
-19979,
-19367,
-18360,
-16980,
-15254,
-13218,
-10913,
-8385,
-5687,
-2873,
0,
2873,
5687,
8385,
10913,
13218,
15254,
16980,
18360,
19367,
19979,
20184,
19979,
19367,
18360,
16980,
15254,
13218,
10913,
8385,
5687,
2873,
0,
-2873,
-5687,
-8385,
-10913,
-13218,
-15254,
-16980,
-18360,
-19367,
-19979,
-20184,
-19979,
-19367,
-18360,
-16980,
-15254,
-13218,

}; */

//pure sine wave
rom const INT16 CURR_SAMP[128] = {
0,
287,
568,
838,
1091,
1321,
1525,
1698,
1836,
1936,
1997,
2018,
1997,
1936,
1836,
1698,
1525,
1321,
1091,
838,
568,
287,
0,
-287,
-568,
-838,
-1091,
-1321,
-1525,
-1698,
-1836,
-1936,
-1997,
-2018,
-1997,
-1936,
-1836,
-1698,
-1525,
-1321,
-1091,
-838,
-568,
-287,
0,
287,
568,
838,
1091,
1321,
1525,
1698,
1836,
1936,
1997,
2018,
1997,
1936,
1836,
1698,
1525,
1321,
1091,
838,
568,
287,
0,
-287,
-568,
-838,
-1091,
-1321,
-1525,
-1698,
-1836,
-1936,
-1997,
-2018,
-1997,
-1936,
-1836,
-1698,
-1525,
-1321,
-1091,
-838,
-568,
-287,
0,
287,
568,
838,
1091,
1321,
1525,
1698,
1836,
1936,
1997,
2018,
1997,
1936,
1836,
1698,
1525,
1321,
1091,
838,
568,
287,
0,
-287,
-568,
-838,
-1091,
-1321,
-1525,
-1698,
-1836,
-1936,
-1997,
-2018,
-1997,
-1936,
-1836,
-1698,
-1525,
-1321,

};

//pure sine wave / 10 - 10A
//rom const INT16 CURR_SAMP[128] = {
//0,
//28,
//56,
//83,
//109,
//132,
//152,
//169,
//183,
//193,
//199,
//201,
//199,
//193,
//183,
//169,
//152,
//132,
//109,
//83,
//56,
//28,
//0,
//-28,
//-56,
//-83,
//-109,
//-132,
//-152,
//-169,
//-183,
//-193,
//-199,
//-201,
//-199,
//-193,
//-183,
//-169,
//-152,
//-132,
//-109,
//-83,
//-56,
//-28,
//0,
//28,
//56,
//83,
//109,
//132,
//152,
//169,
//183,
//193,
//199,
//201,
//199,
//193,
//183,
//169,
//152,
//132,
//109,
//83,
//56,
//28,
//0,
//-28,
//-56,
//-83,
//-109,
//-132,
//-152,
//-169,
//-183,
//-193,
//-199,
//-201,
//-199,
//-193,
//-183,
//-169,
//-152,
//-132,
//-109,
//-83,
//-56,
//-28,
//0,
//28,
//56,
//83,
//109,
//132,
//152,
//169,
//183,
//193,
//199,
//201,
//199,
//193,
//183,
//169,
//152,
//132,
//109,
//83,
//56,
//28,
//0,
//-28,
//-56,
//-83,
//-109,
//-132,
//-152,
//-169,
//-183,
//-193,
//-199,
//-201,
//-199,
//-193,
//-183,
//-169,
//-152,
//-132,
//
//};

//rom const INT16 CURR_SAMP[128] = {
//0,
//2873,
//5687,
//8385,
//10913,
//13218,
//15254,
//16980,
//18360,
//19367,
//19979,
//20184,
//-20184,
//-19979,
//-19367,
//-18360,
//-16980,
//-15254,
//-13218,
//-10913,
//-8385,
//-5687,
//-2873,
//0,
//2873,
//5687,
//8385,
//10913,
//13218,
//15254,
//16980,
//18360,
//19367,
//19979,
//20184,
//19979,
//19367,
//18360,
//16980,
//15254,
//13218,
//10913,
//8385,
//5687,
//2873,
//0,
//-2873,
//-5687,
//-8385,
//-10913,
//-13218,
//-15254,
//-16980,
//-18360,
//-19367,
//-19979,
//-20184,
//-19979,
//-19367,
//-18360,
//-16980,
//-15254,
//-13218,
//-10913,
//-8385,
//-5687,
//-2873,
//0,
//2873,
//5687,
//8385,
//10913,
//13218,
//15254,
//16980,
//18360,
//19367,
//19979,
//20184,
//19979,
//19367,
//18360,
//16980,
//15254,
//13218,
//10913,
//8385,
//5687,
//2873,
//0,
//-2873,
//-5687,
//-8385,
//-10913,
//-13218,
//-15254,
//-16980,
//-18360,
//-19367,
//-19979,
//-20184,
//-19979,
//-19367,
//-18360,
//-16980,
//-15254,
//-13218,
//-13218,
//-10913,
//-8385,
//-5687,
//-2873,
//0,
//2873,
//5687,
//8385,
//10913,
//13218,
//15254,
//16980,
//18360,
//19367,
//19979
//};



