/******************************************************************************

File Name:           RegDef_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#ifndef __RegDef_EnergyMeter
#define __RegDef_EnergyMeter

#include "GenericTypeDefs.h"

//Register definition types
typedef union
{ 
    struct
    {
        unsigned char CycleComplete:1;		//Indicates the Cycle completion for computation
        unsigned char NewLCGroupStart:1;	//Indicates the start of new group of data
        unsigned char PhA_Delay_SignBit:1;	//Indicates if Phase Delayed samples have to be considered in order to implement phase correction
        unsigned char b3:1;					//Not Implemented
        unsigned char b4:1;					//Not Implemented
        unsigned char PLLPhaseLock:1;		//PLL to be enableed or not
        unsigned char CalMode:1;			//Device in CALIBRATION mode or not
        unsigned char DummyVal:1;			//Indicates to use Dummy current and voltage samples for debugging
    } bits;
    unsigned char        byte_val;       // BYTE access

}FLAGS;


typedef union
{ 
    struct
    {
        unsigned char CAL_MODE:1;		//Indicates that CALIBRATION is in progress
        unsigned char CAL_UPDATE:1;		//Indicates CALIBRATION REGISTERS ARE BEING UPDATED
        unsigned char CAL_DUMMYVAL:1;	//Indicates to use dummy calibration values for debugging
        unsigned char b3:1;				//Not Implemented
        unsigned char b4:1;				//Not Implemented
        unsigned char b5:1;				//Not Implemented
        unsigned char b6:1;				//Not Implemented
        unsigned char b7:1;				//Not Implemented
    } bits;
    unsigned char        byte_val;       // BYTE access

}CALB_CONTROL;


typedef union
{ 
    struct
    {
        unsigned char RCDone:1;				//High if comm buffer reception is complete
        unsigned char TXInProgress:1;		//High if comm buffer transmission in progress
        unsigned char RCOverrun:1;			//High if the UART receiver had an overrun
        unsigned char BufOverflow:1;		//High if the receive buffer had an overflow
        unsigned char BufOverrun:1;			//High if the receive buffer had an overrun
        unsigned char DoClrEnergyWZ:1;		//High if ENERGY_W_Z register to be cleared after this read
        unsigned char DoClrEnergyVAZ:1;		//High if ENERGY_VA_Z register to be cleared after this read
        unsigned char b7:1;					//Not Implemented
    } bits;
    unsigned char        byte_val;       	// BYTE access

}COMM_FLAGS;


typedef union
{ 
    struct
    {
        unsigned char :1;							//Not Implemented
        unsigned char STAT1_PHA_W_RAW_SIGN:1;		//Sign of PHA_W_RAW before absolute value taken (if enabled)
        unsigned char :1;							//Not Implemented
        unsigned char :1;							//Not Implemented
        unsigned char :1;							//Not Implemented
        unsigned char :1;							//Not Implemented
        unsigned char :1;							//Not Implemented
        unsigned char :1;							//Not Implemented
    } bits;
    unsigned char        byte_val;       			// BYTE access

}STATUS1_REG;


typedef union
{ 
    struct
    {
        unsigned char MODE1_CREEP:1;			//Creep is enabled if high
        unsigned char MODE1_SING_PHASE:1;		//Multi-point phase corr if low, single point if high
        unsigned char MODE1_ABS:1;				//Take positive energy only if high
        unsigned char MODE1_PHA_W_EN:1;			//Phase A is enabled for energy acc when high
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
    } bits;
    unsigned char        byte_val;       		// BYTE access

}MODE1_REG;


typedef union
{ 
    struct
    {
        unsigned char Curr_Sign:1;				//Sign of  present reactive samples
        unsigned char Eng_Sign:1;				//Sign of energy samples
        unsigned char Power_Sign:1;				//Sign of power samples
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
    } bits;
    unsigned char        byte_val;       		// BYTE access

}REACT_REG;


typedef union _S_DWORD_VAL
{
    struct
    {
        unsigned int LW;
        signed int HW;
    } word;
    struct
    {
        unsigned char LB;
        unsigned char HB;
        unsigned char UB;
        signed char MB;
    } byte;

    struct
    {
        unsigned char b0:1;
        unsigned char b1:1;
        unsigned char b2:1;
        unsigned char b3:1;
        unsigned char b4:1;
        unsigned char b5:1;
        unsigned char b6:1;
        unsigned char b7:1;
        unsigned char b8:1;
        unsigned char b9:1;
        unsigned char b10:1;
        unsigned char b11:1;
        unsigned char b12:1;
        unsigned char b13:1;
        unsigned char b14:1;
        unsigned char b15:1;
        unsigned char b16:1;
        unsigned char b17:1;
        unsigned char b18:1;
        unsigned char b19:1;
        unsigned char b20:1;
        unsigned char b21:1;
        unsigned char b22:1;
        unsigned char b23:1;
        unsigned char b24:1;
        unsigned char b25:1;
        unsigned char b26:1;
        unsigned char b27:1;
        unsigned char b28:1;
        unsigned char b29:1;
        unsigned char b30:1;
        unsigned char b31:1;
    } bits;
} S_DWORD_VAL;


typedef union _S_QWORD_VAL
{
    struct
    {
        unsigned long LD;
        signed long HD;
    } dword;
    struct
    {
        unsigned int LW;
        unsigned int HW;
        unsigned int UW;
        signed int MW;
    } word;
    struct
    {
        unsigned char b0:1;
        unsigned char b1:1;
        unsigned char b2:1;
        unsigned char b3:1;
        unsigned char b4:1;
        unsigned char b5:1;
        unsigned char b6:1;
        unsigned char b7:1;
        unsigned char b8:1;
        unsigned char b9:1;
        unsigned char b10:1;
        unsigned char b11:1;
        unsigned char b12:1;
        unsigned char b13:1;
        unsigned char b14:1;
        unsigned char b15:1;
        unsigned char b16:1;
        unsigned char b17:1;
        unsigned char b18:1;
        unsigned char b19:1;
        unsigned char b20:1;
        unsigned char b21:1;
        unsigned char b22:1;
        unsigned char b23:1;
        unsigned char b24:1;
        unsigned char b25:1;
        unsigned char b26:1;
        unsigned char b27:1;
        unsigned char b28:1;
        unsigned char b29:1;
        unsigned char b30:1;
        unsigned char b31:1;
        unsigned char b32:1;
        unsigned char b33:1;
        unsigned char b34:1;
        unsigned char b35:1;
        unsigned char b36:1;
        unsigned char b37:1;
        unsigned char b38:1;
        unsigned char b39:1;
        unsigned char b40:1;
        unsigned char b41:1;
        unsigned char b42:1;
        unsigned char b43:1;
        unsigned char b44:1;
        unsigned char b45:1;
        unsigned char b46:1;
        unsigned char b47:1;
        unsigned char b48:1;
        unsigned char b49:1;
        unsigned char b50:1;
        unsigned char b51:1;
        unsigned char b52:1;
        unsigned char b53:1;
        unsigned char b54:1;
        unsigned char b55:1;
        unsigned char b56:1;
        unsigned char b57:1;
        unsigned char b58:1;
        unsigned char b59:1;
        unsigned char b60:1;
        unsigned char b61:1;
        unsigned char b62:1;
        unsigned char b63:1;
    } bits;
} S_QWORD_VAL;


typedef union _SQDWORD_VAL
{
    INT8 v[5];

    struct
    {
        INT8 B0;
        INT8 B1;
        INT8 B2;
        INT8 B3;
        INT8 B4;	
    } byte;
} SQDWORD_VAL;


typedef union
{ 
	
    struct
    {
        unsigned char    mcp_configh;       	// MCP CONFIGH value
        unsigned char    mcp_configl;        	// MCP CONFIGL value
        unsigned char    mcp_gain;  	      	// MCP PGA GAIN value
        unsigned char    mcp_status;    	  	// MCP STATUS
    }config_val;   
	unsigned int		 w[2];                  				
    unsigned long        Val;          			// 32 bits access
    
}SDMCP_CON; 


typedef union
{ 
	
    struct
    {
        unsigned char    clk_sec;       	// MCP CONFIGH value
        unsigned char    clk_min;        	// MCP CONFIGL value
    }time;   
	unsigned char		 b[2];                  				
    unsigned int         Val;          			// 32 bits access
    
}Sw_Rtcc; 


typedef union
{ 
    struct
    {
        unsigned char act_blink:1;				//Indicates the status of Active energy blinking. If set then blink occured recently
        unsigned char react_blink:1;			//Indicates the status of Reactive energy blinking. If set then blink occured recently
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
    } bits;
    unsigned char        byte_val;       		// BYTE access

}NO_LOAD_BLINK;


typedef union
{ 
    struct
    {
        unsigned char sine_64_complete:1;		//Indicates 64 sine cycles are completed and need to check MCP3901 config values
        unsigned char mcp_con_chk:1;			//Indicates MCP3901 config values are checked and need not be done till next cycle
        unsigned char mcp_con_err:1;			//Indicates that there is an error in MCP3901  Configuration value and needs reconfiguration
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
        unsigned char :1;						//Not Implemented
    } bits;
    unsigned char        byte_val;       		// BYTE access

}MCP_CONFIG_CHK;

typedef enum
{
	LCD_ActiveEnergy = 0,
	LCD_ReactiveEnergy,
	LCD_Error
}DISPLAY_TYPE;

// File contains the global declarations of all variables used across project

extern FLAGS	        Flags1;

extern MODE1_REG		MODE1;

extern STATUS1_REG		STATUS1;

extern CALB_CONTROL	    CAL_CONTROL;

extern UINT16          LINE_CYC;
extern UINT16          LINE_CYC_CNT;

extern QWORD_VAL       RAW2_I_RMS;
extern UINT16	       RAW_I_RMS;
extern UINT16          I_RMS;

extern QWORD_VAL       RAW2_V_RMS;
extern UINT16	       RAW_V_RMS;
extern UINT16          V_RMS;

extern UINT16	        FREQUENCY;
extern UINT32          PERIOD;

extern QWORD_VAL       RAW_POWER_ACT;
extern UINT32          POWER_ACT;

extern UINT32          POWER_APP;

extern QWORD_VAL       RAW_POWER_REACT;
extern UINT32          POWER_REACT;

extern QWORD_VAL       RAW_ENERGY_ACT;

extern QWORD_VAL       RAW_ENERGY_APP;

extern QWORD_VAL       RAW_ENERGY_REACT;

extern UINT8           I_ABS_MAX;
extern UINT8           V_ABS_MAX;

//********************* CALIBRATION REGISTERS ************************************************************
extern INT8	       	   PHASE_COMPENSATION;
extern INT8            PHASE_COMPENSATION_90;

extern INT16	       OFFSET_I_RMS;
extern UINT16	       GAIN_I_RMS;

extern INT16           OFFSET_V_RMS;
extern UINT16          GAIN_V_RMS;

extern INT32	       OFFSET_POWER_ACT;
extern UINT16	       GAIN_POWER_ACT;
extern UINT16          GAIN_ENERGY_ACT;
extern UINT16          GAIN_NUMR_ENERGY_ACT;
extern UINT8           GAIN_DENR_ENERGY_ACT;

extern INT32	       OFFSET_POWER_REACT;
extern UINT16	       GAIN_POWER_REACT;
extern UINT16          GAIN_ENERGY_REACT;
extern UINT16          GAIN_NUMR_ENERGY_REACT;
extern UINT8           GAIN_DENR_ENERGY_REACT;

extern UINT16          GAIN_ENERGY_APP;

extern UINT8	       CREEP_THRESHOLD_MINUTE;
extern UINT8	       CREEP_THRESHOLD_SECOND;

extern UINT16          METER_CONSTANT;

extern UINT8	       CF_PULSE_WIDTH;

extern UINT16	       MODE1_DEF;
extern UINT16          CAL_STATUS;

extern UINT16	       MAXIMUM_CURRENT;
extern UINT16          CALIBRATION_VOLTAGE;
extern UINT16          CALIBRATION_CURRENT;
extern UINT16          CALIBRATION_FREQUENCY;
extern UINT16          CALIBRATION_LINE_CYCLE;

extern UINT32        ENERGY_ACT;
extern UINT32        ENERGY_APP;
extern UINT32        ENERGY_REACT;

extern UINT8         METER_CONSTANT_BY_100;

extern UINT8         PHASE_COMPENSATION_90_FREQ[];

extern UINT8         LineCyclesRem;
extern UINT8         SaveEnergyStatus;
extern UINT32        Tmr_Seconds_since_EEP_Save;

extern BOOL          EEPROM_Load_Error;

extern UINT16        Signal_Frequency;

extern UINT32        Incremental_CF_Energy_Act;

extern UINT32        Incremental_CF_Energy_React;

extern UINT8         PhaseComp_Amplitude,PhaseComp_Amplitude_90Lag;

extern QWORD_VAL     Summation_Active_Ins_Power,Summation_Reactive_Ins_Power;
extern QWORD_VAL     Summation_RMS_Square_Voltage,Summation_RMS_Square_Current;

extern MCP_CONFIG_CHK	Mcp_Con_Check;
extern SDMCP_CON     MCP_Config;

extern NO_LOAD_BLINK Blink_Energy;
extern Sw_Rtcc       Sw_Clk_Act,Sw_Clk_React;

extern UINT8         VoltageABSMaxAmp,CurrentABSMaxAmp;

extern QWORD_VAL     square_val;
extern UINT32         sq_offset;

extern COMM_FLAGS	 CommFlags;

extern UINT8         *CommsBuffer;
extern UINT8	     *temp_ptr;
extern UINT8         *temp_init,*scr_pad;

extern UINT8         CF_Pulse_Width_W;
extern UINT8         CF_Pulse_Width_W_Rx;
extern UINT16        Period_W;
extern UINT8         LastLineCycCnt,LineCyclesRem;

extern UINT16	     SqrRoot;

extern QWORD_VAL Average_Active_Power;
extern QWORD_VAL Average_Reactive_Power;

extern QWORD_VAL Accumulate_Raw_RMS_Voltage;
extern QWORD_VAL Accumulate_Raw_RMS_Current;

extern QWORD_VAL Active_Energy_Raw;
extern QWORD_VAL Reactive_Energy_Raw;

extern UINT8 temp1,temp2,temp3;
extern QWORD_VAL CF_Acc_Temp,CF_Acc_Temp_Reactive;
extern UINT8 TempyVar0;

extern UINT8    Save_BSR;

extern UINT16    Gain_Reactive_Energy_Frequency[22];
extern UINT16    Gain_Reactive_Power_Frequency[22];
extern QWORD_VAL Reactive_Energy_Raw_Over_One_Cycle;
extern QWORD_VAL Active_Energy_Raw_Over_One_Cycle;

extern BOOL		 GainChangeStatus;
extern REACT_REG Sign_Reg;

extern UINT32 Period_F;
extern UINT16    VAR_ENG_GAIN_FREQUENCY;
extern UINT16    VAR_POW_GAIN_FREQUENCY;

extern QWORD_VAL Accumulate_Active_Energy_Raw;
extern QWORD_VAL Accumulate_Reactive_Energy_Raw;

extern UINT32  Energy_Act_Per_Meter_Constant;
extern UINT32  Energy_React_Per_Meter_Constant;

extern UINT32        Incremental_CF_Energy_Act;
extern UINT32        Incremental_CF_Energy_React;
extern QWORD_VAL  Accumulate_Active_Ins_Power,Accumulate_Reactive_Ins_Power;

//function declarations
void Init_EEPROM_SPI(void);
void SaveEnergyOnPowerDown(void);
void SaveEnergyEvery24Hours(void);
void Store_EN_To_EEP_On_WDT_Reset(void);
BOOL LoadFromEEPROM(UINT16 EEPROM_Adr,UINT8* RAM_Adr,UINT8 num_bytes);
void StoreToEEPROM(UINT16 EEPROM_Adr,UINT8* RAM_Adr,UINT8 num_bytes);


void Init_Delay(void);
void Init_EnergyMeter(void);
void ConfigCalibrationRegsToDefault(void);
void ConfigOpeartionalRegs(void);
void LCD_Update(void);

void Check_SD_ADC_Config(void);

void Init_USART(void);
void ProcessGUICommand(void);

void ADD_64(QWORD_VAL* Data1, QWORD_VAL* Data2, QWORD_VAL* Res);
void SUB_64(QWORD_VAL* Data1, QWORD_VAL* Data2, QWORD_VAL* Res);
void MUL_S_32(INT32 arg1, INT32 arg2, INT16* ptr);
void MUL_U_32(UINT32 arg1, UINT32 arg2, UINT16* ptr);
void MUL_S_64(QWORD_VAL* arg1, UINT16 arg2, QWORD_VAL* ptr);
UINT16 Compute_RMS(void);

void Computation_Cycle_Start(void);
#if defined(RMS_VOLTAGE_ENABLE_COMPUTATION)
void ComputeRMSVoltage(void);
#endif
#if defined(RMS_CURRENT_ENABLE_COMPUTATION)
void ComputeRMSCurrent(void);
#endif
#if defined(ACTIVE_POWER_ENABLE_COMPUTATION)
void ComputeActivePower(void); 
#endif
#if defined(REACTIVE_POWER_ENABLE_COMPUTATION)
void ComputeReactivePower(void); 
#endif
#if defined(APPARENT_POWER_ENABLE_COMPUTATION)
void ComputeApparentPower(void); 
#endif
#if defined(ACTIVE_ENERGY_ENABLE_COMPUTATION)
void ComputeActiveEnergy(void); 
#endif
#if defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
void ComputeReactiveEnergy(void);  
#endif

void Process_SDADC_Data(void);

UINT8 Init_MCP_SD_ADC(UINT8 Gain);
void Init_ADC_SPI(void);

void DisplayDataOnLCD(UINT32 Data,DISPLAY_TYPE Type);
void LCD_Bin_2_BCD(UINT32 Data,UINT8* pointer_lcd);
void Display_LCD(UINT8* LCD_Pattern,DISPLAY_TYPE Type);
void Init_LCD(void);
void Display_Error(void);

#endif	//__RegDef_EnergyMeter

