/******************************************************************************

File Name:           system_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#ifndef __system_EnergyMeter
#define __system_EnergyMeter

#include "HardwareProfile_EnergyMeter.h"
#include "RegDef_EnergyMeter.h"


/****************************************************************************
  Macro:   ClearInterruptStatusSDADC()

 Description:   Macro clears interrupt status of Sigma Delta ADC Data Ready signal through INT0
                         External interrupt.

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define ClearInterruptStatusSDADC()                   INTCONbits.INT0IF = 0

/****************************************************************************
  Macro:   ClearInterruptSDADC()

 Description:   Macro clears interrupt status of Sigma Delta ADC Data Ready signal through INT0
                         External interrupt.

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define ClearInterruptSDADC()                       _asm bcf		INTCON,1,0 _endasm

/****************************************************************************
  Macro:   EnableInterruptSDADC()

 Description:   Macro enables interrupt of Sigma Delta ADC Data Ready signal through INT0
                         External interrupt.

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define EnableInterruptSDADC()                   INTCONbits.INT0IE = 1

/****************************************************************************
  Macro:   DisableInterruptSDADC()

 Description:   Macro disables interrupt of Sigma Delta ADC Data Ready signal through INT0
                         External interrupt.

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define DisableInterruptSDADC()                   INTCONbits.INT0IE = 0

/****************************************************************************
  Macro:   ClearInterruptStatusPowerDownDetect()

 Description:   Macro clears interrupt status of Low Voltage Detect (LVD).

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define ClearInterruptStatusPowerDownDetect()        PIR2bits.LVDIF = 0

/****************************************************************************
  Macro:   EnableInterruptPowerDownDetect()

 Description:   Macro enables interrupt of Low Voltage Detect (LVD) for Power down detection

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define EnableInterruptPowerDownDetect()             PIE2bits.LVDIE = 1

/****************************************************************************
  Macro:   DisableInterruptPowerDownDetect()

 Description:   Macro disables interrupt of Low Voltage Detect (LVD) for Power down detection

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define DisableInterruptPowerDownDetect()             PIE2bits.LVDIE = 0

/****************************************************************************
  Macro:   SystemReset()

 Description:   Macro resets the controller with a software reset instruction

  Precondition:   None

  Parameters:   None

  Remarks: In reset status register, this reset manifests as software reset.
  ***************************************************************************/
#define SystemReset()                                _asm reset	_endasm

/****************************************************************************
  Macro:   IsInterruptFromSDADC()

 Description:   Macro returns the interrupt status of Sigma Delta ADC Data Ready signal through INT0
                         External interrupt.

  Precondition:   INT0 must be initialized

  Parameters:   None
  
  Returns:   Interrupt status of External interrupt INT0 connected to data ready of SD-ADC
                   1 - if INT0 interrupt has occured
                   0 - if INT0 interrupt has not occured

  Remarks: None
  ***************************************************************************/
#define IsInterruptFromSDADC()			INTCONbits.INT0IF


/****************************************************************************
  Macro:   IsInterruptFromPowerDownDatect()

 Description:   Macro returns the interrupt status of Low Voltage Detect (LVD). If power down condition is detected,
                         then LVD interrupt gets generated

  Precondition:   LVD interrupt must be initialized

  Parameters:   None
  
  Returns:   Interrupt status of Power Down Detect LVD interrupt
                   1 - if INT0 interrupt has occured
                   0 - if INT0 interrupt has not occured

  Remarks: None
  ***************************************************************************/
#define IsInterruptFromPowerDownDatect()			PIR2bits.LVDIF

/****************************************************************************
  Macro:   SamplesPerSignalCycle

 Description:   number of sample per signal cycle after which all accumulations are processed

  Remarks: None
  ***************************************************************************/
#define SamplesPerSignalCycle			128


/****************************************************************************
  Macro:   IsInterruptFromSWTimer()

 Description:   Macro returns the interrupt status of TIMER0 interrupt flag. TIMER 0 is used as Software
                         Timer for calculating various Timer tracking purpose

  Precondition:   TIMER0 must be initialized with:
                            * 1:256 prscale
                            * Interrupt set to low priority
                            * 16 bit read write mode

  Parameters:   None
  
  Returns:   Interrupt status of TIMER0 flag
                   1 - if TIMER0 interrupt has occured
                   0 - if TIMER0 interrupt has not occured

  Remarks: None
  ***************************************************************************/
#define IsInterruptFromSWTimer()			INTCONbits.TMR0IF


/****************************************************************************
  Macro:   ClearInterruptStatusSWTimer()

 Description:   Macro clears interrupt status of TIMER0 used as SW Clcok.

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define ClearInterruptStatusSWTimer()			INTCONbits.TMR0IF = 0

/****************************************************************************
  Macro:   IsInterruptFromGUIRXInterface()

 Description:   Macro returns the interrupt status of UART reception interrupt flag. UART is used for Calibration
                         GUI interface

  Precondition:   UART must be configured

  Parameters:   None
  
  Returns:   Interrupt status of UART reception flag
                   1 - if UART reception interrupt has occured
                   0 - if UART reception interrupt has not occured

  Remarks: None
  ***************************************************************************/
#define IsInterruptFromGUIRXInterface()			PIR1bits.RCIF


/****************************************************************************
  Macro:   EnableTransmissionToGUI()

 Description:   Macro enables interrupt for UART transmission

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define EnableTransmissionToGUI()			PIE1bits.TXIE = 1


/****************************************************************************
  Macro:   DisableTransmissionToGUI()

 Description:   Macro disables interrupt for UART transmission

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define DisableTransmissionToGUI()			PIE1bits.TXIE = 0

 
#endif	//__system_EnergyMeter

